//
// Volume Data Visualizer for Google Earth
//
// errorcode.h
//
// Copyright (c) 2012-2015 Shintaro KAWAHARA (kawahara@jamstec.go.jp)
// Japan Agency for Marine-Earth Science and Technology (JAMSTEC)
// http://www.jamstec.go.jp
//
// This program is free software.
// You can redistribute it and/or modify it under the terms of the
// GNU General Public License version 3.0 as published by the Free
// Software Foundation and appearing in the file LICENSE.GPL included
// in the packaging of this file.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef ERRORCODE_H
#define ERRORCODE_H

#include <QString>

enum ErrorCode {
	GRADS_NO_ERROR,
	GRADS_ERROR_CTLFILE_OPEN,
	GRADS_ERROR_DATAFILE_OPEN,
	GRADS_ERROR_MEM_ALOOC,
	GRADS_ERROR_DSET,
	GRADS_ERROR_XDEF,
	GRADS_ERROR_YDEF,
	GRADS_ERROR_ZDEF,
	GRADS_ERROR_TDEF,
	GRADS_ERROR_VARS,
	GRADS_ERROR_FILEHEADER,
	GRADS_ERROR_OPTIONS,
	GRADS_ERROR_UNDEF,
	GRADS_ERROR_DTYPE,
	GRADS_ERROR_CHSUB,
	GRADS_ERROR_INDEX,
	GRADS_ERROR_STNMAP,
	GRADS_ERROR_UNPACK,
	GRADS_ERROR_THEADER,
	GRADS_ERROR_HEADERBYTES,
	GRADS_ERROR_TRAILERBYTES,
	GRADS_ERROR_XYHEADER,
	GRADS_ERROR_XVAR,
	GRADS_ERROR_YVAR,
	GRADS_ERROR_ZVAR,
	GRADS_ERROR_STID,
	GRADS_ERROR_TVAR,
	GRADS_ERROR_TOFFVAR,
	GRADS_ERROR_CACHESIZE,
	GRADS_ERROR_PDEF,
	GRADS_ERROR_EDEF,
	GRADS_ERROR_VECTORPAIRS,
	GRADS_ERROR_ATMARK,

	PARAM_NO_ERROR,
	PARAM_COLORTABLE_NO_ERROR,

	PARAM_ERROR_SAVEFILE_OPEN,
	PARAM_ERROR_TIME_BEGIN,
	PARAM_ERROR_TIME_END,
	PARAM_ERROR_TIME_SKIP,
	PARAM_ERROR_TIME_CONVERSION_TO_UTC,
	PARAM_ERROR_ZUNIT,
	PARAM_ERROR_ZSCALE,
	PARAM_ERROR_OBJECT_X,
	PARAM_ERROR_OBJECT_Y,
	PARAM_ERROR_OBJECT_Z,
	PARAM_ERROR_OBJECT_PITCH,
	PARAM_ERROR_OBJECT_YAW,
	PARAM_ERROR_OBJECT_X_SPHERE,
	PARAM_ERROR_OBJECT_Y_SPHERE,
	PARAM_ERROR_OBJECT_Z_SPHERE,
	PARAM_ERROR_OBJECT_PITCH_SPHERE,
	PARAM_ERROR_OBJECT_YAW_SPHERE,
	PARAM_ERROR_VOLUME_THRESHOLD_MIN,
	PARAM_ERROR_VOLUME_THRESHOLD_MAX,
	PARAM_ERROR_LOGSCALE,
	PARAM_ERROR_VOLUME_X,
	PARAM_ERROR_VOLUME_Y,
	PARAM_ERROR_VOLUME_Z,
	PARAM_ERROR_VOLUME_X_SLICES,
	PARAM_ERROR_VOLUME_Y_SLICES,
	PARAM_ERROR_VOLUME_Z_SLICES,
	PARAM_ERROR_COLORSLICE_X,
	PARAM_ERROR_COLORSLICE_Y,
	PARAM_ERROR_COLORSLICE_Z,
	PARAM_ERROR_COLORSLICE_X_POS,
	PARAM_ERROR_COLORSLICE_Y_POS,
	PARAM_ERROR_COLORSLICE_Z_POS,
	PARAM_ERROR_COLORSLICE_CLAMP_TO_GROUND,
	PARAM_ERROR_COLORSLICE_ALPHA,
	PARAM_ERROR_COLORSLICE_OPACITY,
	PARAM_ERROR_CONTOURLINE_X,
	PARAM_ERROR_CONTOURLINE_Y,
	PARAM_ERROR_CONTOURLINE_Z,
	PARAM_ERROR_CONTOURLINE_X_POS,
	PARAM_ERROR_CONTOURLINE_Y_POS,
	PARAM_ERROR_CONTOURLINE_Z_POS,
	PARAM_ERROR_CONTOURLINE_THRESHOLD_MIN,
	PARAM_ERROR_CONTOURLINE_THRESHOLD_MAX,
	PARAM_ERROR_CONTOURLINE_INTERVAL,
	PARAM_ERROR_CONTOURLINE_WIDTH,
	PARAM_ERROR_BACKGROUND_COLOR,
	PARAM_ERROR_BOUNDINGBOX,
	PARAM_ERROR_BOUNDINGBOX_AXISLABEL,
	PARAM_ERROR_COLORBAR,
	PARAM_ERROR_COLORBAR_DIRECTION,
	PARAM_ERROR_COLORBAR_TYPE,
	PARAM_ERROR_COLORBAR_DECIMAL_PLACES,
	PARAM_ERROR_COLORBAR_MAJOR_TICKMARKS,
	PARAM_ERROR_COLORBAR_MINOR_TICKMARKS,
	PARAM_ERROR_COLORBAR_MAJOR_TICKMARK_TYPE,
	PARAM_ERROR_COLORBAR_MINOR_TICKMARK_TYPE,
	PARAM_ERROR_TIMEDISPLAY,
	PARAM_ERROR_LOGO,
	PARAM_ERROR_LOGOFILE_OPEN,
	PARAM_ERROR_COLORTABLE,
	PARAM_ERROR_COLORTABLE_FILE_OPEN,
	PARAM_ERROR_COLORTABLE_RED,
	PARAM_ERROR_COLORTABLE_GREEN,
	PARAM_ERROR_COLORTABLE_BLUE,
	PARAM_ERROR_COLORTABLE_ALPHA,
	PARAM_ERROR_COLORTABLE_CONNECTION_TYPE,
	PARAM_ERROR_KML_MAX_IMAGE_SIZE,
	PARAM_ERROR_SYNTAX,

	COASTLINE_NO_ERROR,
	COASTLINE_ERROR_FILE_OPEN,
	COASTLINE_ERROR_SYNTAX,

	APPPARAM_ERROR_MAXTEX3D_W,
	APPPARAM_ERROR_MAXTEX3D_H,
	APPPARAM_ERROR_MAXTEX3D_D,
	APPPARAM_ERROR_USE_NATIVE_DIALOG,
	APPPARAM_ERROR_SYNTAX,
	APPPARAM_NO_ERROR
};

QString getErrString( ErrorCode err );

#endif // ERRORCODE_H
